module UU
  module OS
    module Lang

      # Module providing methods +const_defined?+ and +const_get+ allowing
      # to use name of constant with :: even on Ruby < 2.0.
      module ConstantHelper

        # Checks if constant is defined.
        # @param [String] constant Constant name (may include ::).
        # @param [Class] root Root from which to search for constant (default to Object).
        # @return [TrueClass, FalseClass] True if constant is defined.
        def constant_defined?(constant, root = Object)
          return ConstantHelper.constant_defined?(constant, root)
        end

        # Returns constant value.
        # @param [String] constant Constant name (may include ::).
        # @param [Class] root Root from which to search for constants (default to Object).
        # @return [Object] Constant value.
        # @raise [NameError] If constant is not defined.
        def constant_get(constant, root = Object)
          return ConstantHelper.constant_get(constant, root)
        end

        # @see #constant_defined?
        # @private
        def self.constant_defined?(constant, root = Object) # :nodoc:
          modules = constant.split('::')
          modules.each do |mod|
            return false if !root.const_defined?(mod)
            root = root.const_get(mod)
          end
          return true
        end

        # @see #constant_get
        # @private
        def self.constant_get(constant, root = Object) # :nodoc:
          modules = constant.split('::')
          modules.each do |mod|
            raise NameError, "uninitialized constant #{mod}"  if !root.const_defined?(mod)
            root = root.const_get(mod)
          end
          return root
        end

      end

    end
  end
end
